package db

import (
	"applet/app/db/model"
	"applet/app/utils/logx"
	"fmt"
	"xorm.io/xorm"
)

// 批量获取订单信息
func DbsOrderRelateFindByIds(eg *xorm.Engine, pvd string, ids []int64) (*[]model.OrdListRelate, error) {
	var ord []model.OrdListRelate
	if err := eg.In("oid", ids).
		Where("pvd = ? and uid>0", pvd).
		Desc("oid").
		Find(&ord); err != nil {
		return nil, logx.Error(err)
	}
	if len(ord) == 0 {
		return nil, nil
	}
	return &ord, nil
}

// 查询用户某时间段内某状态的佣金(ord_id => amount)
func OrderRelateListByTimeByState(Db *xorm.Engine, uid, state, startTime, endTime interface{}) (*map[int64]float64, error) {
	var o []*model.OrdList
	var or []*model.OrdListRelate
	var idAmountMap = make(map[int64]float64)
	// 分佣明细数据
	err := Db.Where("`uid` = ?", uid).Find(&or)
	if err != nil {
		logx.Error(err)
		return nil, err
	}
	if len(or) == 0 {
		return &idAmountMap, nil
	}
	var ids []int64
	for _, item := range or {
		ids = append(ids, item.Oid)
	}
	str := fmt.Sprintf("state in (%s) AND create_at >%s AND create_at < %s", state, startTime, endTime)
	fmt.Println("============test佣金数=================")
	fmt.Println(str)
	// 已结算的订单数据
	err = Db.In("ord_id", ids).Where("state in (?) AND create_at > ? AND create_at < ?", state, startTime, endTime).Find(&o)
	if err != nil {
		logx.Error(err)
		return nil, err
	}
	if len(o) == 0 {
		return &idAmountMap, nil
	}

	// id => amount
	for _, item := range o {
		for _, i := range or {
			if item.OrdId == i.Oid {
				idAmountMap[i.Oid] = i.Amount
			}
		}
	}

	return &idAmountMap, nil
}

func OrderRelateListByOid(Db *xorm.Engine, oid int64, pvd string) ([]*model.OrdListRelate, error) {
	var ol []*model.OrdListRelate
	err := Db.Where("oid=? and pvd=?", oid, pvd).Find(&ol)
	if err != nil {
		return nil, logx.Error(err)
	}
	return ol, nil
}
func OrderRelateListByOidWithLevel(Db *xorm.Engine, oid int64) (*model.OrdListRelate, error) {
	var ol model.OrdListRelate
	has, err := Db.Where("oid=? and level=?", oid, 0).Get(&ol)
	if has == false || err != nil {
		return nil, logx.Error(err)
	}
	return &ol, nil
}
func OrderRelateDeleteByOid(Db *xorm.Engine, oid int64, pvd string) (int64, error) {
	_, err := Db.Where("oid=? and pvd=?", oid, pvd).Delete(model.OrdListRelate{})
	if err != nil {
		return 0, err
	}
	return 1, nil
}
func OrderRelateDeleteByOidbylevel(Db *xorm.Engine, oid int64, pvd string) (int64, error) {
	_, err := Db.Where("oid=? and pvd=? and level>?", oid, pvd, 0).Delete(model.OrdListRelate{})
	if err != nil {
		return 0, err
	}
	return 1, nil
}

func OrderRelateListByUidAndTimeRangeAndPvdAndLevel(Db *xorm.Engine, uid interface{}, pvd string, level int, stime, etime int64) ([]*model.OrdListRelate, error) {
	var ol []*model.OrdListRelate
	var err error
	if level >= 2 { // 间推
		err = Db.Where("uid=? AND pvd=? AND level>=? AND create_at>=? AND create_at<?", uid, pvd, 2, stime, etime).Find(&ol)
	} else { // 0自购 1直推
		err = Db.Where("uid=? AND pvd=? AND level=? AND create_at>=? AND create_at<?", uid, pvd, level, stime, etime).Find(&ol)
	}
	if err != nil {
		return nil, logx.Error(err)
	}
	return ol, nil
}

func OrderRelateListByUid(Db *xorm.Engine, uid interface{}) ([]*model.OrdListRelate, error) {
	var ol []*model.OrdListRelate
	err := Db.Where("uid=?", uid).Desc("create_at").Find(&ol)
	if err != nil {
		return nil, logx.Error(err)
	}
	return ol, nil
}

func OrderRelateListByOids(Db *xorm.Engine, oids []int64) ([]*model.OrdListRelate, error) {
	var ol []*model.OrdListRelate
	err := Db.In("oid", oids).Find(&ol)
	if err != nil {
		return nil, logx.Error(err)
	}
	return ol, nil
}
