package db

import (
	"applet/app/db/model"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"time"
	"xorm.io/xorm"
)

// SysUnionByTaobaoByUseByAuth is 正在使用Taobao且已授权的设置
func SysUnionByTaobaoByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, "taobao")
	var (
		m model.SysUnionSet
	)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = 'taobao' AND is_use = 1 AND auth_status = 1").Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, ":taobao auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, ": auth taobao 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByJdByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	var (
		m model.SysUnionSet
	)
	has, err := Db.Where("provider = 'jd' AND is_use = 1 and is_delete=0").Get(&m)
	if err != nil {
		return nil, err
	}
	if !has {
		return nil, errors.New("找不到联盟设置")
	}
	return &m, nil
}
func JdUnionComm() map[string]string {
	keyArr := map[string]string{
		"app_key":      "444ab671c4d095122916453d09af27fc",
		"access_token": "",
		"app_secret":   "aef3c2745cc44194816d0c89c65b3847",
	}
	return keyArr
}
func SysUnionByAuthAll(Db *xorm.Engine, provider string) (*[]model.SysUnionSet, error) {

	var (
		m []model.SysUnionSet
	)
	err := Db.Where("provider =? AND auth_status = 1 and is_delete=0 and expire_time>?", provider, time.Now().Format("2006-01-02 15:04:05")).Desc("is_use").Find(&m)
	if err != nil {
		return nil, err
	}
	return &m, nil
}
func SysUnionByCsjpAuthAll(Db *xorm.Engine, provider string) (*[]model.SysUnionSet, error) {

	var (
		m []model.SysUnionSet
	)
	err := Db.Where("provider =? and is_delete=0 ", provider).Desc("is_use").Find(&m)
	if err != nil {
		return nil, err
	}
	return &m, nil
}
func SysUnionByMeituanByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, "meituan")
	var (
		m model.SysUnionSet
	)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = 'meituan' AND is_use = 1 ").Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, ":meituan auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, ": auth meituan 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByTikTokByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, "tikTok")
	var (
		m model.SysUnionSet
	)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = 'tikTok' AND is_use = 1 ").Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, ":tikTok auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, ": auth tikTok 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByCsjpByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, "csjp")
	var (
		m model.SysUnionSet
	)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = 'csjp' AND is_use = 1 ").Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, ":tikTok auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, ": auth tikTok 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByKuaishouByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, "kuaishou")
	var (
		m model.SysUnionSet
	)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = 'kuaishou' AND is_use = 1 ").Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, ":kuaishou auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, ": auth tikTok 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByElmByUseByAuth(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, "elm")
	var (
		m model.SysUnionSet
	)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = 'elm' AND is_use = 1 ").Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, ":tikTok auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, ": auth tikTok 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByTikTokByUseByAuthId(Db *xorm.Engine, id string) (*model.SysUnionSet, error) {
	var (
		m model.SysUnionSet
	)
	has, err := Db.Where("provider = 'tikTok' AND id=? ", id).Get(&m)
	if err != nil {
		return nil, err
	}
	if !has {
		return nil, errors.New("找不到联盟设置")
	}

	return &m, nil
}
func SysUnionByTikTokByUseByAuthNoCache(Db *xorm.Engine, masterId string) (*model.SysUnionSet, error) {
	var (
		m model.SysUnionSet
	)

	has, err := Db.Where("provider = 'tikTok' AND is_use = 1 ").Get(&m)
	if err != nil {
		return nil, err
	}
	if !has {
		return nil, errors.New("找不到联盟设置")
	}
	return &m, nil
}

// SysUnionByPVDByUse is 正在使用的设置
func SysUnionByPVDByUse(Db *xorm.Engine, masterId, pvd string) (*model.SysUnionSet, error) {
	var (
		m model.SysUnionSet
	)
	redisKey := fmt.Sprintf(md.UnionSetCacheCfg, masterId, pvd)
	str, err := cache.GetString(redisKey)
	if err != nil || str == "" {
		has, err := Db.Where("provider = ? AND is_use = 1", pvd).Get(&m)
		if err != nil {
			return nil, err
		}
		if !has {
			return nil, errors.New("找不到联盟设置")
		}
		cache.SetJson(redisKey, m, md.CfgCacheTime)
	} else {
		fmt.Println(masterId, pvd, ": auth 缓存获取的:", str)
		err := json.Unmarshal([]byte(str), &m)
		if err != nil {
			return nil, err
		}
		fmt.Println(masterId, pvd, ": auth 解析到的:", utils.SerializeStr(m))
	}

	return &m, nil
}
func SysUnionByPVDByUseAll(Db *xorm.Engine, masterId, pvd string) (*[]model.SysUnionSet, error) {
	var (
		m []model.SysUnionSet
	)

	err := Db.Where("provider = ? AND is_use = 1", pvd).Find(&m)
	if err != nil {
		return nil, err
	}
	return &m, nil
}

// 更新记录
func SysUnionUpdate(Db *xorm.Engine, data *model.SysUnionSet) {
	row, err := Db.ID(data.Id).AllCols().Update(data)
	if err != nil || row == 0 {
		_ = logx.Warn(err)
	}
}
func SysUnionById(eg *xorm.Engine, id string) *model.SysUnionSet {
	var data model.SysUnionSet
	get, err := eg.Where("id=?", id).Get(&data)
	if get == false || err != nil {
		return nil
	}
	return &data
}
