package db

import (
	"applet/app/db/model"
	"applet/app/utils/logx"
	"time"
	"xorm.io/xorm"
)

func UserVirtualAmountFindByIdWithSession(session *xorm.Session, uid, coinId int) (*model.UserVirtualAmount, error) {
	var m model.UserVirtualAmount
	has, err := session.Where("uid = ? AND coin_id = ?", uid, coinId).Get(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	if has == false {
		m.Amount = "0"
		m.CoinId = coinId
		m.Uid = uid
		one, err := session.InsertOne(&m)
		if err != nil || one == 0 {
			return nil, logx.Warn(err)
		}
	}
	return &m, nil
}
func UserVirtualAmountFindById(eg *xorm.Engine, uid, coinId int) (*model.UserVirtualAmount, error) {
	var m model.UserVirtualAmount
	has, err := eg.Where("uid = ? AND coin_id = ?", uid, coinId).Get(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	if has == false {
		m.Amount = "0"
		m.CoinId = coinId
		m.Uid = uid
		one, err := eg.InsertOne(&m)
		if err != nil || one == 0 {
			return nil, logx.Warn(err)
		}
	}
	return &m, nil
}
func UserVirtualAmountUpdateWithSession(session *xorm.Session, uid, coinId interface{}, userVirtualAmount *model.UserVirtualAmount, forceCols ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceCols != nil {
		affected, err = session.Where("uid = ? AND coin_id = ?", uid, coinId).Cols(forceCols...).Update(userVirtualAmount)
	} else {
		affected, err = session.Where("uid = ? AND coin_id = ?", uid, coinId).Update(userVirtualAmount)
	}
	if err != nil {
		return 0, logx.Warn(err)
	}
	return affected, nil
}
func UserVirtualAmountUpdate(eg *xorm.Engine, uid, coinId interface{}, userVirtualAmount *model.UserVirtualAmount, forceCols ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceCols != nil {
		affected, err = eg.Where("uid = ? AND coin_id = ?", uid, coinId).Cols(forceCols...).Update(userVirtualAmount)
	} else {
		affected, err = eg.Where("uid = ? AND coin_id = ?", uid, coinId).Update(userVirtualAmount)
	}
	if err != nil {
		return 0, logx.Warn(err)
	}
	return affected, nil
}
func UserVirtualCoinFlowInsertOne(eg *xorm.Engine, m *model.UserVirtualCoinFlow) error {
	_, err := eg.InsertOne(m)
	if err != nil {
		return err
	}
	return nil
}
func UserVirtualCoinFlowInsertOneWithSession(session *xorm.Session, m *model.UserVirtualCoinFlow) error {
	_, err := session.InsertOne(m)
	if err != nil {
		return err
	}
	return nil
}

func GetUserVirtualCoinAmount(eg *xorm.Engine, uid int, coinId string) *model.UserVirtualAmount {
	var data model.UserVirtualAmount
	get, err := eg.Where("uid=? and coin_id=?", uid, coinId).Get(&data)
	if get == false || err != nil {
		return nil
	}
	return &data
}

func UserVirtualCoinAmountUpdate(Db *xorm.Engine, id interface{}, userProfile *model.UserVirtualAmount, forceCols ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceCols != nil {
		affected, err = Db.Where("id=?", id).Cols(forceCols...).Update(userProfile)
	} else {
		affected, err = Db.Where("id=?", id).AllCols().Update(userProfile)
	}
	if err != nil {
		return 0, logx.Warn(err)
	}
	return affected, nil
}
func UserVirtualCoinFlowInsert(eg *xorm.Engine, uid int, coinId int, ordId string, ItemTitle string, ordType int, amount string, beforeAmount string, afterAmount string) {

	var data = model.UserVirtualCoinFlow{
		Uid:          uid,
		CoinId:       coinId,
		Direction:    2,
		Title:        ItemTitle,
		OrdId:        ordId,
		Amout:        amount,
		BeforeAmout:  beforeAmount,
		AfterAmout:   afterAmount,
		SysFee:       "",
		CreateTime:   time.Now(),
		TransferType: ordType,
		CoinIdTo:     0,
	}
	eg.Insert(&data)
	return
}
func NewUserVirtualCoinFlowInsert(eg *xorm.Engine, uid, direction int, coinId int, ordId string, ItemTitle string, ordType int, amount string, beforeAmount string, afterAmount string) {

	var data = model.UserVirtualCoinFlow{
		Uid:          uid,
		CoinId:       coinId,
		Direction:    direction,
		Title:        ItemTitle,
		OrdId:        ordId,
		Amout:        amount,
		BeforeAmout:  beforeAmount,
		AfterAmout:   afterAmount,
		SysFee:       "",
		CreateTime:   time.Now(),
		TransferType: ordType,
		CoinIdTo:     0,
	}
	eg.Insert(&data)
	return
}
