package taobao

import (
	"encoding/json"
	"errors"
	"fmt"
	"strings"
	"time"

	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
)

type OrderInfo struct {
	AdzoneID                           int64       `json:"adzone_id"`
	AdzoneName                         string      `json:"adzone_name"`
	AlimamaRate                        string      `json:"alimama_rate"`
	AlimamaShareFee                    string      `json:"alimama_share_fee"`
	AlipayTotalPrice                   string      `json:"alipay_total_price"`
	AppKey                             string      `json:"app_key"`
	ClickTime                          string      `json:"click_time"`
	DepositPrice                       string      `json:"deposit_price"`
	FlowSource                         string      `json:"flow_source"`
	IncomeRate                         string      `json:"income_rate"`
	IsLx                               string      `json:"is_lx"`
	ItemCategoryName                   string      `json:"item_category_name"`
	ItemID                             interface{} `json:"item_id"`
	ItemImg                            string      `json:"item_img"`
	ItemLink                           string      `json:"item_link"`
	ItemNum                            int         `json:"item_num"`
	ItemPrice                          string      `json:"item_price"`
	ItemTitle                          string      `json:"item_title"`
	OrderType                          string      `json:"order_type"`
	PubID                              int         `json:"pub_id"`
	PubShareFee                        string      `json:"pub_share_fee"`
	PubSharePreFee                     string      `json:"pub_share_pre_fee"`
	PubShareRate                       string      `json:"pub_share_rate"`
	RefundTag                          int         `json:"refund_tag"`
	RelationID                         int64       `json:"relation_id"`
	SellerNick                         string      `json:"seller_nick"`
	SellerShopTitle                    string      `json:"seller_shop_title"`
	SiteID                             int         `json:"site_id"`
	SiteName                           string      `json:"site_name"`
	SpecialID                          int64       `json:"special_id"`
	SubsidyFee                         string      `json:"subsidy_fee"`
	SubsidyRate                        string      `json:"subsidy_rate"`
	SubsidyType                        string      `json:"subsidy_type"`
	TbDepositTime                      string      `json:"tb_deposit_time"`
	TbPaidTime                         string      `json:"tb_paid_time"`
	TerminalType                       string      `json:"terminal_type"`
	TkCommissionFeeForMediaPlatform    string      `json:"tk_commission_fee_for_media_platform"`
	TkCommissionPreFeeForMediaPlatform string      `json:"tk_commission_pre_fee_for_media_platform"`
	TkCommissionRateForMediaPlatform   string      `json:"tk_commission_rate_for_media_platform"`
	TkCreateTime                       string      `json:"tk_create_time"`
	TkDepositTime                      string      `json:"tk_deposit_time"`
	TkOrderRole                        int         `json:"tk_order_role"`
	TkPaidTime                         string      `json:"tk_paid_time"`
	TkStatus                           int         `json:"tk_status"`
	TkTotalRate                        string      `json:"tk_total_rate"`
	TotalCommissionFee                 string      `json:"total_commission_fee"`
	TotalCommissionRate                string      `json:"total_commission_rate"`
	TradeID                            string      `json:"trade_id"`
	TradeParentID                      string      `json:"trade_parent_id"`
	ModifiedTime                       string      `json:"modified_time"`
	TkEarningTime                      string      `json:"tk_earning_time"`
}

// 淘宝跟单, 可以通过链接拼装&relationId(int)或者&externalId(string)(二选一)进行订单获取
func (t *TB) OrderTrackSC(pageNo, pageSize int, sTime, eTime int64, arg map[string]string, nextPositionIndex string) (*[]md.OrderInfo, string, error) {

	// 默认查询20分钟内数据
	//sTime = utils.TimeStdParseUnix("2024-03-13 17:42:11")
	//eTime = sTime + 120
	args := arg
	st, et := time.Unix(sTime, 0).Format("2006-01-02 15:04:05"), time.Unix(eTime, 0).Format("2006-01-02 15:04:05")
	queryType, scene := "1", "1"
	if v, ok := args["query_type"]; ok {
		qType := utils.StrToInt(v)
		if qType > 0 && qType <= 4 {
			queryType = v
		}
	}
	// 这里去除了常规订单, 因为常规订单无法进行跟单
	if v, ok := args["order_scene"]; ok {
		sce := utils.StrToInt(v)
		if sce > 0 && sce < 4 {
			scene = v
		}
	}
	//fmt.Printf("******************sid: %v, ak: %s, sk: %v*******************", t.Svc_SID, t.Svc_AK, t.Svc_SK)
	method := "taobao.tbk.sc.order.details.get"
	params := map[string]string{
		"session":        t.SID,
		"start_time":     st,        // 2019-04-05 12:18:22
		"end_time":       et,        // 订单查询结束时间，订单开始时间至订单结束时间，中间时间段日常要求不超过3个小时，但如618、双11、年货节等大促期间预估时间段不可超过20分钟，超过会提示错误，调用时请务必注意时间段的选择，以保证亲能正常调用！
		"query_type":     queryType, // 查询时间类型，1：创建时间，2:付款时间，3:结算时间 4.订单更新时间
		"order_scene":    scene,     // 场景订单场景类型，1:常规订单，2:渠道订单，3:会员运营订单，默认为1
		"page_no":        utils.IntToStr(pageNo),
		"page_size":      utils.IntToStr(pageSize),
		"position_index": nextPositionIndex,
	}
	if args["tk_status"] != "" {
		params["tk_status"] = args["tk_status"]
	}
	resp, err := send(params, method, t.Svc_AK, t.Svc_SK)
	fmt.Println(string(resp))
	if strings.Contains(string(resp), "Invalid session") {
		return nil, "", errors.New("Invalid session")
	}
	if err != nil {
		return nil, "", logx.Warn(err)
	}
	tmp := struct {
		TbkScOrderDetailsGetResponse struct {
			Data struct {
				HasNext       bool   `json:"has_next"`
				HasPre        bool   `json:"has_pre"`
				PageNo        int    `json:"page_no"`
				PageSize      int    `json:"page_size"`
				PositionIndex string `json:"position_index"`
				Results       struct {
					PublisherOrderDto []OrderInfo `json:"publisher_order_dto"`
				} `json:"results"`
			} `json:"data"`
		} `json:"tbk_sc_order_details_get_response"`
	}{}

	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, "", logx.Warn(resp)
	}
	if len(tmp.TbkScOrderDetailsGetResponse.Data.Results.PublisherOrderDto) == 0 {
		return &[]md.OrderInfo{}, "", nil
	}
	positionIndex := tmp.TbkScOrderDetailsGetResponse.Data.PositionIndex
	var orderInfo []md.OrderInfo
	for _, v := range tmp.TbkScOrderDetailsGetResponse.Data.Results.PublisherOrderDto {
		state, reason := ordStatusConvert(v.TkStatus)
		etime := time.Now().Unix()
		if state == md.OrderStatePvdSettle {
			etime = utils.TimeStdParseUnix(v.ModifiedTime)
		}
		orderType := 0
		pid := "self_" + utils.Int64ToStr(v.SpecialID)
		// 判断分享订单
		if utils.Int64ToStr(v.RelationID) != "0" && utils.Int64ToStr(v.RelationID) != "" {
			orderType = 1
			pid = "share_" + utils.Int64ToStr(v.RelationID)
		}
		//判断是不是纯渠道自购的的 判断是否活动订单 如果是活动订单 要以活动推广位判断
		if orderType == 1 && (utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.ActivitySelfPromoIdWeb) || utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.WebRelationSelfKey) || utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.AndroidRelationSelfKey) || utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.IosRelationSelfKey)) {
			orderType = 3
			pid = "actself_" + utils.Int64ToStr(v.RelationID)
		}
		if orderType == 1 && (utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.CloudPromoIdWeb)) {
			orderType = 10
			pid = "cloud_" + utils.Int64ToStr(v.RelationID)
		}
		//判断是否小程序活动订单 如果是活动订单 要以活动推广位判断
		if orderType == 1 && utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.WechatActivitySelfPromoIdWeb) {
			orderType = 4
			pid = "wechatactself_" + utils.Int64ToStr(v.RelationID)
		}
		if orderType == 1 && strings.Contains(t.AppletPid, utils.Int64ToStr(v.AdzoneID)) {
			orderType = 9
			pid = "wechatactselfnew_" + utils.Int64ToStr(v.RelationID) + "_" + utils.Int64ToStr(v.AdzoneID)
		}
		//判断是否小程序活动订单 如果是活动订单 要以活动推广位判断
		if orderType == 1 && utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.WechatActivitySharePromoIdWeb) {
			orderType = 5
			pid = "wechatactshare_" + utils.Int64ToStr(v.RelationID)
		}
		// 判断免单
		if orderType == 0 && utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SelfPromoIdFree) {
			orderType = 2
			pid = "free_" + utils.Int64ToStr(v.SpecialID)
		}
		if orderType == 1 && utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SelfPromoIdRelationFree) {
			orderType = 15
			pid = "relationfree_" + utils.Int64ToStr(v.RelationID)
		}
		// 判断淘礼金免单
		if orderType == 0 && utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SelfPromoIdTljFree) {
			orderType = 6
			pid = "tljfree_" + utils.Int64ToStr(v.SpecialID)
		}
		if orderType == 0 && utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SelfPromoIdRelationTljFree) {
			orderType = 16
			pid = "ttljfree_" + utils.Int64ToStr(v.RelationID)
		}
		// 判断淘礼金渠道免单
		if utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SelfPromoIdTljRelationFree) {
			orderType = 7
			pid = "tljrelationfree_" + utils.Int64ToStr(v.RelationID)
		}
		//判断是分享的
		if (utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SharePromoIdWeb) || utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SharePromoIdAndroid) || utils.Int64ToStr(v.AdzoneID) == AdZoneId(t.SharePromoIdIOS)) && utils.Int64ToStr(v.AdzoneID) != "" && utils.Int64ToStr(v.AdzoneID) != "0" {
			orderType = 1
			pid = "share_" + utils.Int64ToStr(v.RelationID)
		}
		//如果都0就清空推广位
		if v.SpecialID == 0 && v.RelationID == 0 {
			pid = "self"
		}
		fmt.Println(pid)
		// 检查是否比价订单
		isCompare := 0
		if v.FlowSource == "自主推广" {
			isCompare = 1
		}
		//判断是否饿了么订单
		pvd := md.PVD_TB
		if v.OrderType == "饿了么" {
			pvd = "ele"
		}

		// 调整佣金计算
		totalCommission := "0.00"
		if v.PubShareFee != "" && v.PubShareFee != "0.00" && v.PubShareFee != "0" {
			totalCommission = v.PubShareFee
		} else if v.PubSharePreFee != "" && v.PubSharePreFee != "0.00" && v.PubSharePreFee != "0" {
			totalCommission = v.PubSharePreFee
		}
		itemID, ok := v.ItemID.(float64)
		if ok {
			v.ItemID = int64(itemID)
		}
		goodsId := utils.AnyToString(v.ItemID)
		if goodsId == "" || goodsId == "<nil>" {
			goodsId = v.TradeID
		}
		fmt.Println("======================", v.ItemID)
		fmt.Println("======================", goodsId)
		utils.FilePutContents("tb_order", v.TradeID)

		orderInfo = append(orderInfo, md.OrderInfo{
			UnionId:        t.UnionId,
			Pvd:            pvd,
			PvdOid:         v.TradeID,                         // TradeParentID总订单号, Trade子订单
			ParentOrdId:    utils.StrToInt64(v.TradeParentID), // TradeParentID总订单号, Trade子订单
			ItemId:         goodsId,
			ItemNum:        v.ItemNum,
			ItemPrice:      utils.StrToFloat64(v.ItemPrice),
			ItemTitle:      v.ItemTitle,
			ItemLink:       v.ItemLink,
			Thumbnail:      v.ItemImg,
			Pid:            pid,
			Commission:     utils.StrToFloat64(totalCommission),
			CommissionRate: utils.StrToFloat64(v.TotalCommissionRate) / 100,
			Reason:         reason,
			PayTime:        utils.TimeParseStd(v.TbPaidTime).Unix(),
			StartTime:      utils.TimeParseStd(v.TkCreateTime).Unix(),
			EndTime:        etime,
			PaidPrice:      utils.StrToFloat64(v.AlipayTotalPrice),
			OrderType:      orderType, // 判断是自购分享还是
			IsCompare:      isCompare,
			State:          state,
			ConfirmAt:      utils.TimeParseStd(v.TkEarningTime).Unix(),
		})
	}
	fmt.Println(orderInfo)
	return &orderInfo, positionIndex, err
}

func ordStatusConvert(s int) (state int, reason string) {
	// 1:"全部订单",3:"订单结算",12:"订单付款", 13:"订单失效",14:"订单成功", 12-付款，13-关闭，14-确认收货，3-结算成功
	stateArr := map[int]string{1: "买家超时未付款", 2: "取消了订单", 3: "订单结算", 12: "订单付款", 13: "订单失效", 14: "订单成功"}
	switch s {
	case 1:
		return md.OrderStateFail, stateArr[1]
	case 2:
		return md.OrderStateFail, stateArr[2]
	case 3:
		return md.OrderStatePvdSettle, ""
	case 12:
		return md.OrderStatePaid, ""
	case 13:
		return md.OrderStateFail, stateArr[13]
	case 14:
		return md.OrderStateSucc, ""
	}
	return 13, "订单失效"
}

/* 返回格式
参照文档 https://open.taobao.com/api.htm?docId=43755&docType=2
{
    "tbk_sc_order_details_get_response": {
        "data": {
            "has_next": true,
            "has_pre": false,
            "page_no": 1,
            "page_size": 1,
            "position_index": "1610349202_1tPBXKBv64y2|1610349202_1tPBXKBv64y2",
            "results": {
                "publisher_order_dto": [
                    {
                        "adzone_id": 97352550314,
                        "adzone_name": "嗨如意渠道分享",
                        "alimama_rate": "0.00",
                        "alimama_share_fee": "0.00",
                        "app_key": "26302019",
                        "click_time": "2021-01-11 15:13:09",
                        "deposit_price": "0.00",
                        "flow_source": "--",
                        "income_rate": "16.25",
                        "is_lx": "0",
                        "item_category_name": "医疗器械",
                        "item_id": 625635920121,
                        "item_img": "//img.alicdn.com/tfscom/i1/2208725638729/O1CN01zaLtlx2ELvtjNrrbC_!!2208725638729.jpg",
                        "item_link": "http://item.taobao.com/item.htm?id=625635920121",
                        "item_num": 1,
                        "item_price": "298.00",
                        "item_title": "振德医疗一次性医用外科口罩医护灭菌型三层防护独立包装含99熔喷",
                        "order_type": "天猫",
                        "pub_id": 13406079,
                        "pub_share_fee": "0.00",
                        "pub_share_pre_fee": "0.00",
                        "pub_share_rate": "100.00",
                        "refund_tag": 0,
                        "relation_id": 528397833,
                        "seller_nick": "振德延天龄专卖店",
                        "seller_shop_title": "振德延天龄专卖店",
                        "site_id": 15484476,
                        "site_name": "嗨如意",
                        "subsidy_fee": "0.00",
                        "subsidy_rate": "0.00",
                        "subsidy_type": "--",
                        "tb_deposit_time": "--",
                        "terminal_type": "无线",
                        "tk_commission_fee_for_media_platform": "0.00",
                        "tk_commission_pre_fee_for_media_platform": "0.00",
                        "tk_commission_rate_for_media_platform": "0.00",
                        "tk_create_time": "2021-01-11 15:13:22",
                        "tk_deposit_time": "--",
                        "tk_order_role": 2,
                        "tk_status": 13,    # 已付款：指订单已付款，但还未确认收货 已收货：指订单已确认收货，但商家佣金未支付 已结算：指订单已确认收货，且商家佣金已支付成功 已失效：指订单关闭/订单佣金小于0.01元，订单关闭主要有：1）买家超时未付款； 2）买家付款前，买家/卖家取消了订单；3）订单付款后发起售中退款成功；3：订单结算，12：订单付款， 13：订单失效，14：订单成功
                        "tk_total_rate": "16.25",
                        "total_commission_fee": "0.00",
                        "total_commission_rate": "16.25",
                        "trade_id": "1243143914745388298",
                        "trade_parent_id": "1243143914745388298"
                    }
                ]
            }
        },
        "request_id": "6jo66wukt1iw"
    }
}

*/
