package zhimeng

import (
	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
)

var cinemaState = map[string]int{
	"订单付款": 0,
	"订单成功": 2,
	"订单完成": 2,
	"订单失效": 4,
	"订单退款": 4,
	"订单结算": 3,
}

func CinemaOrderTrack(appKey, appSecret string, pageNo, pageSize int, sTime, eTime int64) (*[]md.OrderInfo, error) {

	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize), // 智盟限定100,超过报错
		"start_time": utils.Int64ToStr(sTime),  // 时间戳
		"end_time":   utils.Int64ToStr(eTime),  // 时间戳
		"is_zyos":    "1",
	}
	fmt.Println(params)
	resp, err := Send("kfc_cinema", "get_cinema_order", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int    `json:"success"`
		Msg     string `json:"msg"`
		Data    []struct {
			ID           string `json:"id"`
			Oid          string `json:"oid"`
			Info         string `json:"goods_title"`
			Commission   string `json:"commission"`
			UID          string `json:"zuid"`
			Payment      string `json:"payment"`
			Status       string `json:"status"`
			CreateTime   string `json:"create_time"`
			ZmUpdateTime string `json:"zm_update_time"`
			GoodsImg     string `json:"goods_img"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	if len(tmp.Data) == 0 {
		return nil, nil
	}

	var (
		res       []md.OrderInfo
		state     int
		reason    string
		orderType int
	)
	for _, v := range tmp.Data {

		state = cinemaState[v.Status]
		if state == md.OrderStateFail {
			reason = v.Status
		}
		res = append(res, md.OrderInfo{
			PvdOid:     v.Oid,
			Pvd:        md.PVD_CINEMA,
			Pid:        "self_" + v.UID,
			ItemNum:    1,
			ItemPrice:  utils.AnyToFloat64(v.Payment),
			ItemTitle:  v.Info,
			PaidPrice:  utils.AnyToFloat64(v.Payment),
			OrderType:  orderType, // 默认为自购
			State:      state,
			Commission: utils.StrToFloat64(v.Commission),
			Reason:     reason,
			PayTime:    utils.StrToInt64(v.CreateTime),
			StartTime:  utils.StrToInt64(v.CreateTime),
			EndTime:    utils.StrToInt64(v.ZmUpdateTime),
			Thumbnail:  v.GoodsImg,
		})
	}
	return &res, nil
}
