package zhimeng

import (
	"applet/app/e"
	"applet/app/lib/plan"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
)

func ElmOrderTrack(appKey, appSecret string, pageNo, pageSize int, sTime, eTime int64, types, dbName, op string, opt *plan.PlanOpt) (*[]md.OrderInfo, error) {
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize), // 智盟限定100,超过报错
		"start_time": utils.Int64ToStr(sTime),  // 时间戳
		"end_time":   utils.Int64ToStr(eTime),  // 时间戳
		"type":       types,
		"mid":        dbName,
	}
	fmt.Println(params)
	resp, err := Send("elm_lm_ord", op, params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int    `json:"success"`
		Msg     string `json:"msg"`
		Data    []struct {
			ID                    string `json:"id"`
			Oid                   string `json:"oid"`
			ParentOid             string `json:"parent_oid"`
			Info                  string `json:"goods_title"`
			Commission            string `json:"commission"`
			SettleCommission      string `json:"settle_commission"`
			PlatformCommissionFee string `json:"platform_commission_fee"`
			UID                   string `json:"zuid"`
			Payment               string `json:"payment"`
			Status                string `json:"state"`
			CreateTime            string `json:"create_time"`
			ZmUpdateTime          string `json:"update_time"`
			GoodsImg              string `json:"goods_img"`
			IsShare               string `json:"is_share"`
			ProductNum            string `json:"product_num"`
		} `json:"data"`
	}
	fmt.Println(string(resp))

	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	if len(tmp.Data) == 0 {
		return nil, nil
	}

	var (
		res       []md.OrderInfo
		state     int
		reason    string
		orderType int
	)
	for _, v := range tmp.Data {

		state = oilstationState[v.Status]
		if state == md.OrderStateFail {
			reason = v.Status
		}
		pid := "self_" + v.UID
		if v.IsShare == "1" {
			pid = "share_" + v.UID
			orderType = 1
		}
		if v.IsShare == "2" {
			pid = "free_" + v.UID
			orderType = 2
		}
		if utils.StrToFloat64(v.SettleCommission) > 0 {
			v.Commission = v.SettleCommission
		}

		if utils.StrToFloat64(v.PlatformCommissionFee) == 0 && opt.OrderBeforeRate > 0 {
			v.PlatformCommissionFee = utils.Float64ToStr(utils.StrToFloat64(v.Commission) * (opt.OrderBeforeRate / 100))
			if utils.StrToFloat64(v.PlatformCommissionFee) < 0.01 {
				v.PlatformCommissionFee = "0"
			}
		}
		res = append(res, md.OrderInfo{
			PvdOid:      v.Oid,
			ParentOrdId: utils.StrToInt64(v.ParentOid),
			Pvd:         "elm",
			Pid:         pid,
			ItemNum:     utils.StrToInt(v.ProductNum),
			ItemPrice:   utils.AnyToFloat64(v.Payment),
			ItemTitle:   v.Info,
			PaidPrice:   utils.AnyToFloat64(v.Payment),
			OrderType:   orderType, // 默认为自购
			State:       state,
			Commission:  utils.StrToFloat64(v.Commission) - utils.StrToFloat64(v.PlatformCommissionFee),
			Reason:      reason,
			PayTime:     utils.StrToInt64(v.CreateTime),
			StartTime:   utils.StrToInt64(v.CreateTime),
			EndTime:     utils.StrToInt64(v.ZmUpdateTime),
			Thumbnail:   v.GoodsImg,
			ItemId:      v.Oid,
		})
	}
	return &res, nil
}
