package zhimeng

import (
	"encoding/json"
	"strings"

	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
)

// promoId一般使用自购ID拼装起来
func JDRefundOrderTrack(appKey, appSecret, webID, dbName string) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeParseStd("2021-04-24 11:32:04").Unix()
	//eTime = sTime + 3600
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"dbName":     dbName,
		"is_zyos":    "1",
	}
	resp, err := Send("jd", "jd_new_refund", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Msg     string `json:"msg"`
		Success int    `json:"success"`
		Data    []struct {
			Oid              string `json:"oid"`
			StatusStr        string `json:"status_str"`
			Gid              string `json:"gid"`
			GoodsTitle       string `json:"goods_title"`
			Commission       string `json:"commission"`
			Payment          string `json:"payment"`
			Fukuan           string `json:"fukuan"`
			Shixiao          string `json:"shixiao"`
			Jiesuan          string `json:"jiesuan"`
			Pid              string `json:"pid"`
			CreateTime       string `json:"create_time"`
			PaymentTime      string `json:"payment_time"`
			GoodsImg         string `json:"goods_img"`
			CommissionRate   string `json:"commission_rate"`
			UpdateTime       string `json:"update_time"`
			DomainID         string `json:"domain_id"`
			JiesuanTime      string `json:"jiesuan_time"`
			JdPid            string `json:"jd_pid"`
			DzTime           string `json:"dz_time"`
			ShareApp         string `json:"share_app"`
			ShareAppExtendID string `json:"share_app_extend_id"`
			GoodsInfo        []struct {
				GoodsPrice string `json:"goods_price"`
				Gid        string `json:"gid"`
				GoodsTitle string `json:"goods_title"`
				GoodsNum   string `json:"goods_num"`
			} `json:"goods_info"`
			GoodsPrice interface{} `json:"goods_price"`
			GoodsNum   interface{} `json:"goods_num"`
			Type       string      `json:"type"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}

	var (
		res       []md.OrderInfo
		reason    string
		orderType int
	)
	for _, v := range tmp.Data {
		if !strings.Contains(v.Pid, "share") && !strings.Contains(v.Pid, "self") && !strings.Contains(v.Pid, "free") {
			continue
		}
		// 分享
		if strings.Contains(v.Pid, "share_") {
			orderType = md.OrderTypeShare
		}
		// 新人免单
		if strings.Contains(v.Pid, "free_") {
			orderType = md.OrderTypeFree
		}
		if v.StatusStr != "订单部分退款" && v.StatusStr != "订单退款" {
			// 跳过待付款的订单
			continue
		}
		state := 6
		if v.StatusStr == "订单退款" {
			state = 4
		}
		if utils.StrToInt64(v.PaymentTime) == 0 {
			v.PaymentTime = v.CreateTime
		}
		//if utils.StrToInt64(v.JiesuanTime) > 0 {
		//	state = md.OrderStatePvdSettle
		//	v.UpdateTime = v.JiesuanTime
		//}
		res = append(res, md.OrderInfo{
			PvdOid:         v.Oid,
			Pvd:            md.PVD_JD,
			ItemId:         v.Gid,
			ItemNum:        len(v.GoodsInfo),
			ItemPrice:      utils.AnyToFloat64(v.Payment),
			ItemTitle:      v.GoodsTitle,
			ItemLink:       "https://item.m.jd.com/product/" + v.Gid + ".html",
			Thumbnail:      getImage(appKey, appSecret, webID, v.Gid, v.GoodsImg),
			PaidPrice:      utils.AnyToFloat64(v.Payment),
			OrderType:      orderType, // 默认为自购
			Pid:            v.Pid,     // 推广位ID
			State:          state,
			Commission:     utils.StrToFloat64(v.Commission),
			CommissionRate: utils.StrToFloat64(v.CommissionRate),
			Reason:         reason,
			PayTime:        utils.StrToInt64(v.PaymentTime),
			StartTime:      utils.StrToInt64(v.CreateTime),
			EndTime:        utils.StrToInt64(v.UpdateTime),
		})
	}
	return &res, nil
}
