package zhimeng

import (
	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
)

var meituanLmState = map[string]int{
	"订单付款": 0,
	"订单成功": 2,
	"订单完成": 2,
	"订单失效": 4,
	"订单退款": 4,
	"订单结算": 3,
}
var meituanLmType = map[string]string{
	"0": "meituan_shop",
	"4": "meituan_takeout",
	"2": "meituan_hotel",
	"6": "meituan_supermarket",
	"8": "meituan_preferred",
}

func MeituanLmOrderTrack(appKey, appSecret string, pageNo, pageSize int, sTime, eTime int64) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeParseStd("2022-08-11 22:41:13").Unix()
	//eTime = sTime + 120
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize), // 智盟限定100,超过报错
		"start_time": utils.Int64ToStr(sTime),  // 时间戳
		"end_time":   utils.Int64ToStr(eTime),  // 时间戳
		"is_zyos":    "1",
	}
	fmt.Println(params)
	resp, err := Send("meituan_lm", "get_updateorder", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int    `json:"success"`
		Msg     string `json:"msg"`
		Data    []struct {
			ID           string `json:"id"`
			Oid          string `json:"oid"`
			Info         string `json:"goods_title"`
			Commission   string `json:"commission"`
			UID          string `json:"zuid"`
			Payment      string `json:"payment"`
			Status       string `json:"status"`
			CreateTime   string `json:"create_time"`
			JsTime       string `json:"js_time"`
			IsShare      string `json:"is_share"`
			ZmUpdateTime string `json:"zm_update_time"`
			OrderType    string `json:"platform"`
			ProductName  string `json:"product_name"`
			ChildOid     string `json:"child_oid"`
		} `json:"data"`
	}
	fmt.Println(json.Unmarshal(resp, &tmp))

	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}

	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	if len(tmp.Data) == 0 {
		return nil, nil
	}

	var (
		res       []md.OrderInfo
		state     int
		reason    string
		orderType int
	)
	for _, v := range tmp.Data {
		// 分享
		var Pid = "self_" + v.UID
		if v.IsShare == "1" {
			orderType = md.OrderTypeShare
			Pid = "share_" + v.UID
		}
		state = meituanLmState[v.Status]
		if state == md.OrderStateFail {
			reason = v.Status
		}
		types := "meituan"
		if meituanLmType[v.OrderType] != "" {
			types = meituanLmType[v.OrderType]
		}
		res = append(res, md.OrderInfo{
			ParentOrdId: utils.StrToInt64(v.Oid),
			PvdOid:      v.ChildOid,
			Pvd:         types,
			Pid:         Pid,
			ItemNum:     1,
			ItemPrice:   utils.AnyToFloat64(v.Payment),
			ItemTitle:   v.Info,
			PaidPrice:   utils.AnyToFloat64(v.Payment),
			OrderType:   orderType, // 默认为自购
			State:       state,
			Commission:  utils.StrToFloat64(v.Commission),
			Reason:      reason,
			PayTime:     utils.StrToInt64(v.CreateTime),
			StartTime:   utils.StrToInt64(v.CreateTime),
			EndTime:     utils.StrToInt64(v.ZmUpdateTime),
		})
	}
	fmt.Println(res)

	return &res, nil
}

func MeituanLmOrderStartTrack(appKey, appSecret string, pageNo, pageSize int, sTime, eTime int64) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeParseStd("2021-03-12 16:07:08").Unix()
	//eTime = sTime + 3600
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize), // 智盟限定100,超过报错
		"start_time": utils.Int64ToStr(sTime),  // 时间戳
		"end_time":   utils.Int64ToStr(eTime),  // 时间戳
		"is_zyos":    "1",
	}
	fmt.Println(params)
	resp, err := Send("meituan_lm", "get_startorder", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int    `json:"success"`
		Msg     string `json:"msg"`
		Data    []struct {
			ID           string `json:"id"`
			Oid          string `json:"oid"`
			Info         string `json:"goods_title"`
			Commission   string `json:"commission"`
			UID          string `json:"zuid"`
			Payment      string `json:"payment"`
			Status       string `json:"status"`
			CreateTime   string `json:"create_time"`
			JsTime       string `json:"js_time"`
			IsShare      string `json:"is_share"`
			ZmUpdateTime string `json:"zm_update_time"`
			OrderType    string `json:"platform"`
			ProductName  string `json:"product_name"`
			ChildOid     string `json:"child_oid"`
		} `json:"data"`
	}
	fmt.Println(json.Unmarshal(resp, &tmp))

	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}

	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	if len(tmp.Data) == 0 {
		return nil, nil
	}

	var (
		res       []md.OrderInfo
		state     int
		reason    string
		orderType int
	)
	for _, v := range tmp.Data {
		// 分享
		var Pid = "self_" + v.UID
		if v.IsShare == "1" {
			orderType = md.OrderTypeShare
			Pid = "share_" + v.UID
		}
		state = meituanLmState[v.Status]
		if state == md.OrderStateFail {
			reason = v.Status
		}
		types := "meituan"
		if meituanLmType[v.OrderType] != "" {
			types = meituanLmType[v.OrderType]
		}
		res = append(res, md.OrderInfo{
			ParentOrdId: utils.StrToInt64(v.Oid),
			PvdOid:      v.ChildOid,
			Pvd:         types,
			Pid:         Pid,
			ItemNum:     1,
			ItemPrice:   utils.AnyToFloat64(v.Payment),
			ItemTitle:   v.Info,
			PaidPrice:   utils.AnyToFloat64(v.Payment),
			OrderType:   orderType, // 默认为自购
			State:       state,
			Commission:  utils.StrToFloat64(v.Commission),
			Reason:      reason,
			PayTime:     utils.StrToInt64(v.CreateTime),
			StartTime:   utils.StrToInt64(v.CreateTime),
			EndTime:     utils.StrToInt64(v.ZmUpdateTime),
		})
	}
	fmt.Println(res)

	return &res, nil
}
