package zhimeng

import (
	"encoding/json"
	"fmt"
	"strings"

	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
)

var pddState = map[string]int{
	"已成团":  md.OrderStatePaid,
	"已收货":  md.OrderStateRecv,
	"确认收货": md.OrderStateRecv,
	"已完成":  md.OrderStateSucc,
	"已结算":  md.OrderStatePvdSettle,
	"审核通过": md.OrderStatePaid,
	"取消":   md.OrderStateFail,
	"审核失败": md.OrderStateFail,
	"订单退款": md.OrderStateFail,
}

// promoId一般使用自购ID拼装起来
func PddOrderTrack(appKey, appSecret string, pageNo, pageSize int, sTime, eTime int64, option, dbName, types string) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeStdParseUnix("2021-07-05 08:15:10")
	//eTime = sTime + 3600
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize),
		"start_time": utils.Int64ToStr(sTime), // 这里是秒的时间戳
		"end_time":   utils.Int64ToStr(eTime), // 这里是秒的时间戳
		"domain_id":  "9",
		"dbName":     dbName,
		"is_zyos":    "1",
		"type":       types,
	}
	fmt.Println(params)
	resp, err := Send("pdd", option, params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Msg     string `json:"msg"`
		Success int    `json:"success"`
		Data    []struct {
			ID               string      `json:"id"`
			Oid              string      `json:"oid"`
			Gid              string      `json:"gid"`
			AppKey           string      `json:"app_key"`
			GoodsTitle       string      `json:"goods_title"`
			Commission       string      `json:"commission"`
			Platform         string      `json:"platform"`
			UID              string      `json:"uid"`
			Payment          string      `json:"payment"`
			StatusStr        string      `json:"status_str"`
			Fukuan           string      `json:"fukuan"`
			Shixiao          string      `json:"shixiao"`
			Jiesuan          string      `json:"jiesuan"`
			Pid              string      `json:"pid"`
			CreateTime       string      `json:"create_time"`
			PaymentTime      string      `json:"payment_time"`
			Fanli            string      `json:"fanli"`
			Append           interface{} `json:"append"`
			FanliTime        string      `json:"fanli_time"`
			PddPid           string      `json:"pdd_pid"`
			GoodsImg         string      `json:"goods_img"`
			CommissionRate   string      `json:"commission_rate"`
			UpdateTime       string      `json:"update_time"`
			DomainID         string      `json:"domain_id"`
			JiesuanTime      string      `json:"jiesuan_time"`
			JdPid            string      `json:"jd_pid"`
			JdYfgIsValid     string      `json:"jd_yfg_is_valid"`
			JdYfgIsDz        string      `json:"jd_yfg_is_dz"`
			DzTime           string      `json:"dz_time"`
			OrderReceiveTime string      `json:"order_receive_time"` //确认收货时间
			CheckSuccessTime string      `json:"check_success_time"` //订单审核时间
			ZmUpdateTime     string      `json:"zm_update_time"`
			ShareApp         string      `json:"share_app"`
			ShareAppExtendID string      `json:"share_app_extend_id"`
			FailStaus        string      `json:"fail_staus"`
			GoodsPrice       float64     `json:"goods_price"`
			GoodsNum         int         `json:"goods_num"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	var (
		res []md.OrderInfo
	)

	for _, v := range tmp.Data {
		var (
			state     int
			reason    string
			orderType int
		)
		if !strings.Contains(v.Pid, "cloud") && !strings.Contains(v.Pid, "share") && !strings.Contains(v.Pid, "self") && !strings.Contains(v.Pid, "free") {
			continue
		}
		if strings.Contains(v.PddPid, "share_") {
			orderType = md.OrderTypeShare
		}
		if strings.Contains(v.PddPid, "cloud_") {
			orderType = 10
		}
		if strings.Contains(v.PddPid, "free_") {
			orderType = md.OrderTypeFree
		}
		state = pddState[v.StatusStr]
		if strings.Contains(v.StatusStr, "取消") {
			state = 4
		}
		reason = ""
		if state == md.OrderStateFail {
			reason = v.StatusStr
		}
		if utils.StrToInt64(v.JiesuanTime) > 0 && state != 4 {
			state = md.OrderStatePvdSettle
			v.UpdateTime = v.JiesuanTime
		}
		orderCompare := 0
		if v.FailStaus == "比价订单" {
			orderCompare = 1
		}
		res = append(res, md.OrderInfo{
			PvdOid:         v.Oid,
			Pvd:            md.PVD_PDD,
			ItemId:         v.Gid,
			ItemNum:        v.GoodsNum,
			ItemPrice:      v.GoodsPrice,
			ItemTitle:      v.GoodsTitle,
			ItemLink:       "https://mobile.yangkeduo.com/goods.html?goods_id=" + v.Gid,
			Thumbnail:      v.GoodsImg,
			PaidPrice:      utils.StrToFloat64(v.Payment),
			OrderType:      orderType, // 默认为自购
			Pid:            v.Pid,     // 推广位ID
			State:          state,
			Commission:     utils.StrToFloat64(v.Commission),
			CommissionRate: utils.StrToFloat64(v.CommissionRate),
			Reason:         reason,
			PayTime:        utils.StrToInt64(v.PaymentTime),
			StartTime:      utils.StrToInt64(v.CreateTime),
			EndTime:        utils.StrToInt64(v.JiesuanTime),
			CheckSuccessAt: utils.StrToInt64(v.CheckSuccessTime),
			ConfirmAt:      utils.StrToInt64(v.OrderReceiveTime),
			IsCompare:      orderCompare,
		})
	}
	return &res, nil
}

// promoId一般使用自购ID拼装起来
func PddOrderMoreTrack(appKey, appSecret string, ids string) (*[]md.OrderInfo, error) {
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"ids":        ids,
	}
	fmt.Println(params)
	resp, err := Send("pdd", "more_order", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Msg     string `json:"msg"`
		Success int    `json:"success"`
		Data    []struct {
			ID               string      `json:"id"`
			Oid              string      `json:"oid"`
			Gid              string      `json:"gid"`
			AppKey           string      `json:"app_key"`
			GoodsTitle       string      `json:"goods_title"`
			Commission       string      `json:"commission"`
			Platform         string      `json:"platform"`
			UID              string      `json:"uid"`
			Payment          string      `json:"payment"`
			StatusStr        string      `json:"status_str"`
			Fukuan           string      `json:"fukuan"`
			Shixiao          string      `json:"shixiao"`
			Jiesuan          string      `json:"jiesuan"`
			Pid              string      `json:"pid"`
			CreateTime       string      `json:"create_time"`
			PaymentTime      string      `json:"payment_time"`
			Fanli            string      `json:"fanli"`
			Append           interface{} `json:"append"`
			FanliTime        string      `json:"fanli_time"`
			PddPid           string      `json:"pdd_pid"`
			GoodsImg         string      `json:"goods_img"`
			CommissionRate   string      `json:"commission_rate"`
			UpdateTime       string      `json:"update_time"`
			DomainID         string      `json:"domain_id"`
			JiesuanTime      string      `json:"jiesuan_time"`
			JdPid            string      `json:"jd_pid"`
			JdYfgIsValid     string      `json:"jd_yfg_is_valid"`
			JdYfgIsDz        string      `json:"jd_yfg_is_dz"`
			DzTime           string      `json:"dz_time"`
			OrderReceiveTime string      `json:"order_receive_time"` //确认收货时间
			CheckSuccessTime string      `json:"check_success_time"` //订单审核时间
			ZmUpdateTime     string      `json:"zm_update_time"`
			ShareApp         string      `json:"share_app"`
			ShareAppExtendID string      `json:"share_app_extend_id"`
			FailStaus        string      `json:"fail_staus"`
			GoodsPrice       float64     `json:"goods_price"`
			GoodsNum         int         `json:"goods_num"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	var (
		res       []md.OrderInfo
		state     int
		reason    string
		orderType int
	)

	for _, v := range tmp.Data {
		state = pddState[v.StatusStr]
		if strings.Contains(v.StatusStr, "取消") {
			state = 4
		}
		if state == md.OrderStateFail {
			reason = v.StatusStr
		}
		if utils.StrToInt64(v.JiesuanTime) > 0 && state != 4 {
			state = md.OrderStatePvdSettle
			v.UpdateTime = v.JiesuanTime
		}
		orderCompare := 0
		if v.FailStaus == "比价订单" {
			orderCompare = 1
		}
		res = append(res, md.OrderInfo{
			PvdOid:         v.Oid,
			Pvd:            md.PVD_PDD,
			ItemId:         v.Gid,
			ItemNum:        v.GoodsNum,
			ItemPrice:      v.GoodsPrice,
			ItemTitle:      v.GoodsTitle,
			ItemLink:       "https://mobile.yangkeduo.com/goods.html?goods_id=" + v.Gid,
			Thumbnail:      v.GoodsImg,
			PaidPrice:      utils.StrToFloat64(v.Payment),
			OrderType:      orderType, // 默认为自购
			Pid:            v.Pid,     // 推广位ID
			State:          state,
			Commission:     utils.StrToFloat64(v.Commission),
			CommissionRate: utils.StrToFloat64(v.CommissionRate),
			Reason:         reason,
			PayTime:        utils.StrToInt64(v.PaymentTime),
			StartTime:      utils.StrToInt64(v.CreateTime),
			EndTime:        utils.StrToInt64(v.JiesuanTime),
			CheckSuccessAt: utils.StrToInt64(v.CheckSuccessTime),
			ConfirmAt:      utils.StrToInt64(v.OrderReceiveTime),
			IsCompare:      orderCompare,
		})
	}
	return &res, nil
}
