package zhimeng

import (
	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
)

func T3carOrderTrack(appKey, appSecret, mid string, types string, pvd string, pageNo, pageSize int, sTime, eTime int64, op string) (*[]md.OrderInfo, error) {
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize), // 智盟限定100,超过报错
		"start_time": utils.Int64ToStr(sTime),  // 时间戳
		"end_time":   utils.Int64ToStr(eTime),  // 时间戳
		"type":       types,
		"mid":        mid,
	}
	fmt.Println(params)
	resp, err := Send("t3car", op, params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int    `json:"success"`
		Msg     string `json:"msg"`
		Data    []struct {
			ID           string `json:"id"`
			Oid          string `json:"oid"`
			Info         string `json:"info"`
			Commission   string `json:"commission"`
			UID          string `json:"zuid"`
			Payment      string `json:"payment"`
			Status       string `json:"status"`
			IsShare      string `json:"is_share"`
			CreateTime   string `json:"create_time"`
			ZmUpdateTime string `json:"zm_update_time"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	if len(tmp.Data) == 0 {
		return nil, nil
	}

	var (
		res       []md.OrderInfo
		state     int
		reason    string
		orderType int
	)
	for _, v := range tmp.Data {

		state = hwState[v.Status]
		if state == md.OrderStateFail {
			reason = v.Status
		}
		pid := "self_" + v.UID
		if v.IsShare == "1" {
			pid = "share_" + v.UID
		}
		res = append(res, md.OrderInfo{
			PvdOid:     v.Oid,
			Pvd:        pvd,
			Pid:        pid,
			ItemId:     v.Oid,
			ItemNum:    1,
			ItemPrice:  utils.AnyToFloat64(v.Payment),
			ItemTitle:  v.Info,
			PaidPrice:  utils.AnyToFloat64(v.Payment),
			OrderType:  orderType, // 默认为自购
			State:      state,
			Commission: utils.StrToFloat64(v.Commission),
			Reason:     reason,
			PayTime:    utils.StrToInt64(v.CreateTime),
			StartTime:  utils.StrToInt64(v.CreateTime),
			EndTime:    utils.StrToInt64(v.ZmUpdateTime),
		})
	}
	return &res, nil
}
