package zhimeng

import (
	"applet/app/e"
	"encoding/json"
	"fmt"

	"applet/app/md"
	"applet/app/utils/logx"
)

//会员id
func TaobaoPublisherInfoList(appkey, appsecret, tbSid, tbAppKey string) (*[]md.PublisherInfo, error) {
	args := map[string]interface{}{
		"appkey":     appkey,
		"secret_key": appsecret,
		"session":    tbSid,
		"tb_app_key": tbAppKey,
	}
	resp, err := Send("agent_tb", "publisher_info_list", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data []struct {
			RelationId string `json:"relation_id"`
			SpecialId  string `json:"special_id"`
			ExternalId string `json:"external_id"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}

	var (
		res []md.PublisherInfo
	)
	for _, v := range tmp.Data {

		res = append(res, md.PublisherInfo{
			RelationId: v.RelationId,
			SpecialId:  v.SpecialId,
			ExternalId: v.ExternalId,
		})
	}
	return &res, nil
}

//会员id
func TaobaoPublisherInfoListByExternalId(appkey, appsecret, tbSid, tbAppKey, externalId string) (*[]md.PublisherInfo, error) {
	args := map[string]interface{}{
		"appkey":      appkey,
		"secret_key":  appsecret,
		"session":     tbSid,
		"tb_app_key":  tbAppKey,
		"external_id": externalId,
	}
	resp, err := Send("agent_tb", "publisher_info_list_by_external_id", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	fmt.Println(string(resp))
	var tmp struct {
		Data []struct {
			RelationId string `json:"relation_id"`
			SpecialId  string `json:"special_id"`
			ExternalId string `json:"external_id"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}

	var (
		res []md.PublisherInfo
	)
	for _, v := range tmp.Data {

		res = append(res, md.PublisherInfo{
			RelationId: v.RelationId,
			SpecialId:  v.SpecialId,
			ExternalId: v.ExternalId,
		})
	}
	return &res, nil
}

//会员id
func TaobaoPublisherRelationInfoList(appkey, appsecret, tbSid, p, tbAppKey string) (*[]md.PublisherInfo, error) {
	args := map[string]interface{}{
		"appkey":     appkey,
		"secret_key": appsecret,
		"session":    tbSid,
		"p":          p,
		"tb_app_key": tbAppKey,
	}
	resp, err := Send("agent_tb", "publisher_info_relation_list", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data []struct {
			RelationId string `json:"relation_id"`
			SpecialId  string `json:"special_id"`
			ExternalId string `json:"external_id"`
			Rtag       string `json:"rtag"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}

	var (
		res []md.PublisherInfo
	)
	for _, v := range tmp.Data {

		res = append(res, md.PublisherInfo{
			RelationId: v.RelationId,
			SpecialId:  v.SpecialId,
			ExternalId: v.ExternalId,
			Rtag:       v.Rtag,
		})
	}
	return &res, nil
}

//绑定的渠道列表
func TaobaoPublisherRelationBindList(appkey, appsecret, tbSid, p, tbAppKey string) (*[]md.PublisherInfo, error) {
	args := map[string]interface{}{
		"appkey":     appkey,
		"secret_key": appsecret,
		"session":    tbSid,
		"p":          p,
		"tb_app_key": tbAppKey,
	}
	resp, err := Send("agent_tb", "publisher_info_relation_list2", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data []struct {
			RelationId string `json:"relation_id"`
			SpecialId  string `json:"special_id"`
			ExternalId string `json:"external_id"`
			Rtag       string `json:"rtag"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}

	var (
		res []md.PublisherInfo
	)
	for _, v := range tmp.Data {

		res = append(res, md.PublisherInfo{
			RelationId: v.RelationId,
			SpecialId:  v.SpecialId,
			ExternalId: v.ExternalId,
			Rtag:       v.Rtag,
		})
	}
	return &res, nil
}
