package zhimeng

import (
	"encoding/json"
	"fmt"
	"strings"

	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
)

type ZmOrderSubInfo struct {
	Gid            string `json:"gid"`
	GoodsImg       string `json:"goods_img"`
	Title          string `json:"title"`
	Payment        string `json:"payment"`
	Commission     string `json:"commission"`
	CommissionRate string `json:"commission_rate"`
}

var vipState = map[string]int{
	"订单付款": 0,
	"订单成功": 2,
	"不合格":  4,
	"已完结":  3,
}

// TODO  未完成数据结构
func VipOrderTrack(appKey, appSecret string, pageNo, pageSize int, sTime, eTime int64, option string) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeParseStd("2021-03-05 12:00:00").Unix()
	//eTime = sTime + 3600
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize), // 智盟限定100,超过报错
		"start_time": utils.Int64ToStr(sTime),  // 时间戳
		"end_time":   utils.Int64ToStr(eTime),  // 时间戳
	}
	resp, err := Send("wph", option, params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int    `json:"success"`
		Msg     string `json:"msg"`
		Data    []struct {
			ID               string           `json:"id"`
			ShareAppExtendID string           `json:"share_app_extend_id"`
			Oid              string           `json:"oid"`
			Gid              string           `json:"gid"`
			Info             string           `json:"info"`
			Commission       string           `json:"commission"`
			CommissionRate   string           `json:"commission_rate"`
			UID              string           `json:"uid"`
			Payment          string           `json:"payment"`
			Status           string           `json:"status"`
			ChannelTag       string           `json:"channelTag"`
			NewCustomer      string           `json:"newCustomer"`
			CreateTime       string           `json:"create_time"`
			SignTime         string           `json:"sign_time"`
			JsTime           string           `json:"js_time"`
			ZmUpdateTime     string           `json:"zm_update_time"`
			StationPid       string           `json:"station_pid"`
			GoodsInfo        []ZmOrderSubInfo `json:"goods_info"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	if len(tmp.Data) == 0 {
		return nil, nil
	}

	var (
		res       []md.OrderInfo
		orderType int
	)
	for _, v := range tmp.Data {
		if !strings.Contains(v.StationPid, "cloud") && !strings.Contains(v.StationPid, "share") && !strings.Contains(v.StationPid, "self") && !strings.Contains(v.StationPid, "free") {
			continue
		}
		// 分享
		if strings.Contains(v.StationPid, "share_") {
			orderType = md.OrderTypeShare
		}
		if strings.Contains(v.StationPid, "cloud_") {
			orderType = 10
		}
		// 新人免单
		if strings.Contains(v.StationPid, "free_") {
			orderType = md.OrderTypeFree
		}
		if v.Status == "待定" {
			// 跳过待付款的订单
			continue
		}
		reason := ""
		state := vipState[v.Status]
		if state == md.OrderStateFail {
			reason = v.Status
		}
		res = append(res, md.OrderInfo{
			PvdOid:         v.Oid,
			Pvd:            md.PVD_VIP,
			ItemId:         v.Gid,
			ItemNum:        len(v.GoodsInfo),
			ItemPrice:      utils.AnyToFloat64(v.Payment),
			ItemTitle:      getVipTitle(v.GoodsInfo),
			ItemLink:       "https://m.vip.com/product-0-" + v.Gid + ".html",
			Thumbnail:      getVipImage(v.GoodsInfo),
			PaidPrice:      utils.AnyToFloat64(v.Payment),
			OrderType:      orderType,    // 默认为自购
			Pid:            v.StationPid, // 推广位ID
			State:          state,
			Commission:     utils.StrToFloat64(v.Commission),
			CommissionRate: utils.StrToFloat64(v.CommissionRate),
			Reason:         reason,
			PayTime:        utils.StrToInt64(v.CreateTime),
			StartTime:      utils.StrToInt64(v.CreateTime),
			EndTime:        utils.StrToInt64(v.ZmUpdateTime),
		})
	}
	return &res, nil
}

func getVipTitle(infos []ZmOrderSubInfo) string {
	var title string
	if len(infos) == 0 {
		return ""
	}
	title = infos[0].Title
	if len(infos) > 1 {
		title = fmt.Sprintf("%s..等共%d件商品", title, len(infos))
	}
	return title
}

func getVipImage(infos []ZmOrderSubInfo) string {
	if len(infos) == 0 {
		return ""
	}
	return infos[0].GoodsImg
}
