package svc

import (
	"applet/app/lib/new_plan"
	"applet/app/utils"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"encoding/json"
	"fmt"
	"strings"

	"xorm.io/xorm"

	"applet/app/db"
	"applet/app/db/model"
	"applet/app/lib/plan"
	"applet/app/md"
	"applet/app/utils/logx"
)

var pvdCfgList = map[string]struct {
	AK  string
	SK  string
	Sid string
}{
	md.PVD_TB:    {"third_taobao_svc_ak", "third_taobao_svc_sk", "third_taobao_web_sid"},
	md.PVD_KL:    {md.KEY_CFG_KL_AK, md.KEY_CFG_KL_SK, ""},
	md.PVD_SN:    {md.KEY_CFG_SN_AK, md.KEY_CFG_SN_SK, ""},
	md.PVD_JD:    {md.KEY_CFG_ZM_AK, md.KEY_CFG_ZM_SK, md.KEY_CFG_ZM_WEB_ID},
	md.PVD_JDOwn: {md.KEY_CFG_ZM_AK, md.KEY_CFG_ZM_SK, md.KEY_CFG_ZM_WEB_ID},
	md.PVD_PDD:   {md.KEY_CFG_ZM_AK, md.KEY_CFG_ZM_SK, md.KEY_CFG_ZM_WEB_ID},
	md.PVD_VIP:   {md.KEY_CFG_ZM_AK, md.KEY_CFG_ZM_SK, md.KEY_CFG_ZM_WEB_ID},
}

func getPlanCfg(eg *xorm.Engine, pvd string) (*plan.PlanOpt, error) {
	opt := &plan.PlanOpt{}
	// 根据供应商
	rewardOpt, err := db.DbsPlanRewardByPvd(eg, pvd)
	if err != nil {
		return nil, err
	}
	if pvd == md.PVD_JDOwn && (&rewardOpt == nil || (rewardOpt != nil && rewardOpt.PlanCommissionId == 0)) {
		rewardOpt, err = db.DbsPlanRewardByPvd(eg, md.PVD_JD)
		if err != nil {
			return nil, err
		}
	}
	if rewardOpt == nil {
		return nil, logx.Warn("找不到方案记录")
	}
	if rewardOpt.State == 0 {
		return nil, logx.Warn("抽成方案未开启")
	}
	if rewardOpt.PlanCommissionId == 0 {
		return nil, logx.Warn("抽成方案未设置佣金方案id")
	}
	commissionOpt, err := db.DbsPlanCommissionById(eg, rewardOpt.PlanCommissionId)
	if err != nil {
		return nil, err
	}
	if _, ok := plan.Fn[commissionOpt.Mode]; !ok && commissionOpt.Mode != "public_platoon" {
		return nil, logx.Warn("分佣模式不存在")
	}
	opt.Pvd = pvd
	opt.Mode = commissionOpt.Mode
	opt.SysRate = float64(int64(rewardOpt.SysRate*1e4)) / 1e4
	opt.PvdRate = float64(int64(rewardOpt.PvdRate*1e4)) / 1e4
	opt.GlobalRate = float64(int64(rewardOpt.GlobalRate*1e4)) / 1e4
	opt.RegionRate = float64(int64(rewardOpt.RegionRate*1e4)) / 1e4
	opt.MerchantRate = float64(int64(rewardOpt.MerchantRate*1e4)) / 1e4
	opt.PushHandRate = float64(int64(rewardOpt.PushHandRate*1e4)) / 1e4
	opt.PlanCommissionId = rewardOpt.PlanCommissionId
	// TODO 结算方案未写入
	if commissionOpt.Mode == "public_platoon" { //公排
		return opt, nil
	}
	if opt.SysRate >= 1 || opt.PvdRate >= 1 || opt.PvdRate+opt.SysRate >= 1 {
		return nil, logx.Warn("分佣方案数据设置总额不能大于1")
	}
	var subsidyTmp map[int]*plan.LvGrade
	commissionOpt.Data = strings.ReplaceAll(commissionOpt.Data, "\"bili\":0", "\"bili\":\"0\"")
	if err := json.Unmarshal([]byte(commissionOpt.Data), &subsidyTmp); err != nil {
		return nil, logx.Warn("分佣方案数据设置错误")
	}
	opt.UserRate = subsidyTmp
	return opt, nil
}

// 获取所有供应商对应的分佣方案, 包括免单,一元购
func PlanOpts(eg *xorm.Engine) map[string]*plan.PlanOpt {
	// 获取所有佣金方案
	allCommissionPlan := db.DbsPlanCommissionByIds(eg)
	if allCommissionPlan == nil {
		return nil
	}
	commissionPlans := map[int]*model.PlanCommission{}
	for _, v := range allCommissionPlan {
		v.Data = strings.ReplaceAll(v.Data, "\"bili\":0", "\"bili\":\"0\"")
		commissionPlans[v.Id] = v
	}

	// 获取所有抽成方案的计划
	allRewardPlan, err := db.DbsPlanRewardByPvds(eg)
	if err != nil || allRewardPlan == nil {
		return nil
	}
	list, _ := db.SecondGetAllPriceTypeList(eg)
	for _, v := range list {
		var oneRewardPlan map[string]string
		err := json.Unmarshal([]byte(v.CommissionData), &oneRewardPlan)
		if err == nil {
			var tmp = &model.PlanReward{
				Id:               0,
				Pvd:              "seFree_" + utils.IntToStr(v.Id),
				PvdRate:          utils.StrToFloat64(oneRewardPlan["pvd_rate"]) / 100,
				SysRate:          utils.StrToFloat64(oneRewardPlan["sys_rate"]) / 100,
				RegionRate:       utils.StrToFloat64(oneRewardPlan["region_rate"]) / 100,
				GlobalRate:       utils.StrToFloat64(oneRewardPlan["global_rate"]) / 100,
				SettleMode:       utils.StrToInt(oneRewardPlan["settle_mode"]),
				PlanCommissionId: utils.StrToInt(oneRewardPlan["plan_commission_id"]),
				PlanSettleId:     utils.StrToInt(oneRewardPlan["plan_settle_id"]),
				State:            1,
				Source:           1,
				IntegralOpen:     utils.StrToInt(oneRewardPlan["integral_open"]),
				MerchantRate:     utils.StrToFloat32(oneRewardPlan["merchant_rate"]) / 100,
				PushHandRate:     utils.StrToFloat32(oneRewardPlan["push_hand_rate"]) / 100,
				OrderBeforeRate:  utils.StrToFloat64(oneRewardPlan["order_before_rate"]) / 100,
			}
			allRewardPlan = append(allRewardPlan, tmp)
		}
	}
	moreList, _ := db.MoreGetAllPriceTypeList(eg)
	for _, v := range moreList {
		var oneRewardPlan map[string]string
		err := json.Unmarshal([]byte(v.CommissionData), &oneRewardPlan)
		if err == nil {
			var tmp = &model.PlanReward{
				Id:               0,
				Pvd:              "moreFree_" + utils.IntToStr(v.Id),
				PvdRate:          utils.StrToFloat64(oneRewardPlan["pvd_rate"]) / 100,
				SysRate:          utils.StrToFloat64(oneRewardPlan["sys_rate"]) / 100,
				RegionRate:       utils.StrToFloat64(oneRewardPlan["region_rate"]) / 100,
				GlobalRate:       utils.StrToFloat64(oneRewardPlan["global_rate"]) / 100,
				SettleMode:       utils.StrToInt(oneRewardPlan["settle_mode"]),
				PlanCommissionId: utils.StrToInt(oneRewardPlan["plan_commission_id"]),
				PlanSettleId:     utils.StrToInt(oneRewardPlan["plan_settle_id"]),
				State:            1,
				Source:           1,
				IntegralOpen:     utils.StrToInt(oneRewardPlan["integral_open"]),
				MerchantRate:     utils.StrToFloat32(oneRewardPlan["merchant_rate"]) / 100,
				PushHandRate:     utils.StrToFloat32(oneRewardPlan["push_hand_rate"]) / 100,
				OrderBeforeRate:  utils.StrToFloat64(oneRewardPlan["order_before_rate"]) / 100,
			}
			allRewardPlan = append(allRewardPlan, tmp)
		}
	}
	opts := map[string]*plan.PlanOpt{}
	for _, v := range allRewardPlan {
		if _, ok := commissionPlans[v.PlanCommissionId]; ok && v.State == 1 && v.PlanCommissionId > 0 {
			var subsidyTmp map[int]*plan.LvGrade
			if err := json.Unmarshal([]byte(commissionPlans[v.PlanCommissionId].Data), &subsidyTmp); err != nil {
				logx.Warn("分佣方案数据设置错误 :", commissionPlans[v.PlanCommissionId])
				continue
			}
			opts[v.Pvd] = &plan.PlanOpt{
				Pvd:              v.Pvd,
				PlanCommissionId: v.PlanCommissionId,
				Mode:             commissionPlans[v.PlanCommissionId].Mode,
				CommissionMode:   commissionPlans[v.PlanCommissionId].CommissionMode,
				IntegralMode:     commissionPlans[v.PlanCommissionId].IntegralMode,
				BlockIconsMode:   commissionPlans[v.PlanCommissionId].BlockIconsMode,
				SysRate:          utils.FloatFormat(v.SysRate, 8),
				PvdRate:          utils.FloatFormat(v.PvdRate, 8),
				RegionRate:       utils.FloatFormat(v.RegionRate, 8),
				RegionSubRate:    utils.FloatFormat(v.RegionSubRate, 8),
				GlobalRate:       utils.FloatFormat(v.GlobalRate, 8),
				UserRate:         subsidyTmp,
				MerchantRate:     utils.FloatFormat(float64(v.MerchantRate), 8),
				OrderBeforeRate:  utils.FloatFormat(v.OrderBeforeRate, 8),
			}

		}
	}
	fmt.Println(opts)

	if len(opts) == 0 {
		return nil
	}
	return opts
}

func GetCreditCardRate(eg *xorm.Engine, user *md.User, opts map[string]*plan.PlanOpt) string {
	opt, ok := opts[md.PVD_CREDIT_CARD]
	if opts == nil || !ok {
		// 方案为空写定45
		return "45"
	}
	var SelfRate float64 = 0
	if opt.Mode == "public_platoon" {
		commissionOpt, err := db.DbsPlanCommissionById(eg, opt.PlanCommissionId)
		if err != nil || commissionOpt == nil || commissionOpt.Id == 0 {
			return "35"
		}
		var subsidyTmp = make(map[string]*new_plan.PublicLvGrade)
		var tmp map[string]*new_plan.PublicLvGrade
		if err := json.Unmarshal([]byte(commissionOpt.Data), &tmp); err != nil {
			return "35"
		}
		for _, v := range tmp {
			subsidyTmp[zhios_order_relate_utils.IntToStr(v.Lv)] = v
		}
		lvGrade := subsidyTmp[utils.IntToStr(user.Info.Level)]
		if lvGrade == nil {
			return "35"
		}
		userRate, ok1 := lvGrade.SelfRateList["0"]
		if !ok1 {
			return "35"
		}
		SelfRate = utils.StrToFloat64(userRate) / 100
	} else {
		userRate, ok1 := opt.UserRate[user.Info.Level]
		if !ok1 {
			return "35"
		}
		SelfRate = userRate.SelfRate
	}

	return utils.AnyToString(((1 - opt.SysRate - opt.PvdRate) * SelfRate) * 100)
}
